# /packages/intranet-agile/www/sprint-planner-action.tcl
#
# Copyright (c) 2003-2010 ]project-open[
#
# All rights reserved. Please check
# https://www.project-open.com/ for licensing details.

ad_page_contract {
    Sprint Planner Action
    Accepts "events" (clicking on an arrow) from the sprint-planner
    and moves the tasks accordingly.
} {
    project_id:integer
    task_id:integer
    action
    return_url
}

ns_log Notice "sprint-planner-action.tcl: project_id=$project_id, task_id=$task_id, action=$action"

# ------------------------------------------------------------
# Defaults & Permissions
# ------------------------------------------------------------

set user_id [auth::require_login]
im_project_permissions $user_id $project_id view read write admin
if {!$write} {
    ad_return_complaint 1 "<li>[_ intranet-core.lt_You_have_insufficient_6]"
    return
}


# ------------------------------------------------------------
# Determine what agile state set to display

set project_type_id [db_string ptype_id "select project_type_id from im_projects where project_id = :project_id" -default 0]
set agile_category_type [db_string category_type_id "select aux_string1 from im_categories where category_id = :project_type_id" -default ""]
if {"" eq $agile_category_type} { 
    ad_return_complaint 1 "Could not determine default agile state range for project_type=[im_category_from_id $project_type_id]"
}
set agile_category_type "Intranet Agile SCRUM States"
set default_agile_status_id 88110

set main_project_id $project_id
set old_task_id $task_id



# ------------------------------------------------------------
# Get information about the affected agile task

set task_info_sql "
	select	coalesce(agile_status_id, :default_agile_status_id) as task_agile_status_id,
		coalesce(t.agile_sort_order, p.sort_order, 0) as task_agile_sort_order,
		coalesce((
			select s.project_id from im_projects s where p.parent_id = s.project_id and s.project_type_id = [im_project_type_scrum]
		), :main_project_id) as task_agile_sprint_id
	from	im_timesheet_tasks t,
		im_projects p
	where	t.task_id = :old_task_id and
		t.task_id = p.project_id
"
set exists_p [db_0or1row task_info $task_info_sql]

if {!$exists_p} {
    # The project doesn't exist
    ad_return_complaint 1 [lang::message::lookup "" intranet-agile.project_does_not_exist "
	<br>
	<b>The specified task #$old_task_id does not exist in project #$main_project_id</b>:<br>
	The task or the project may have been deleted.
	<br>&nbsp;<br>
    "]
} 
ns_log Notice "sprint-planner-action.tcl: task_agile_status_id=$task_agile_status_id, task_agile_sort_order=$task_agile_sort_order, task_agile_sprint_id=$task_agile_sprint_id"



# ------------------------------------------------------------
# Get the list of tasks below the main project
#
set tasks_sql "
	select	t.task_id,
		t.agile_status_id,
		t.agile_sort_order,
		p.project_name as task_name,
		p.project_type_id as task_type_id,
		coalesce((
			select s.project_id from im_projects s where p.parent_id = s.project_id and s.project_type_id = [im_project_type_scrum]
		), :main_project_id) as sprint_id
	from	im_projects main_p,
		im_projects p,
		im_timesheet_tasks t
	where	main_p.project_id = :main_project_id and
		p.tree_sortkey between main_p.tree_sortkey and tree_right(main_p.tree_sortkey) and
		p.project_id = t.task_id and
		not exists (select * from im_projects sub_p where sub_p.parent_id = p.project_id)	-- Only leafs
	order by 
	      	t.agile_sort_order
"

set task_list [list]; # All tasks in the project
set sprint_status_task_list [list]; # Only tasks with status and in sprint of $task_id
db_foreach update_agile_status $tasks_sql {
    lappend task_list $task_id
    if {$sprint_id eq $task_agile_sprint_id && $agile_status_id eq $task_agile_status_id} { lappend sprint_status_task_list $task_id }
    set sort_order_hash($task_id) $agile_sort_order
}
# ad_return_complaint 1 "[im_ad_hoc_query -format "html" $tasks_sql], sprint_status_task_list=$sprint_status_task_list"


# ------------------------------------------------------------
# States
#
set top_states_sql "
	select	category_id,
		category
	from	im_categories c
	where	((c.enabled_p = 't' OR c.enabled_p is NULL) and category_type = :agile_category_type) 
		OR c.category_id in (select agile_status_id from ($tasks_sql) t)
	order by sort_order, category_id
"

set top_states_list [list]
db_foreach top_states $top_states_sql {
    lappend top_states_list $category_id
}
# ad_return_complaint 1 $top_states_list




# ------------------------------------------------------------
# Update the task according to action

# determine the position of the current status_id in the
# list of agile states
set old_agile_status_pos [lsearch $top_states_list $task_agile_status_id]
set old_agile_sort_order_pos [lsearch $sprint_status_task_list $old_task_id]

set new_agile_status_pos $old_agile_status_pos
set new_agile_sort_order_pos $old_agile_sort_order_pos

ns_log Notice "sprint-planner-action.tcl: before action switch: action=$action, old_task_id=$old_task_id, states_list=$top_states_list, sprint_status_task_list=$sprint_status_task_list"


switch $action {
    left {
	set new_agile_status_pos [expr $old_agile_status_pos - 1]
    }
    right {
	set new_agile_status_pos [expr $old_agile_status_pos + 1]
    }
    up {
	set new_agile_sort_order_pos [expr $old_agile_sort_order_pos - 1]
    }
    down {
	set new_agile_sort_order_pos [expr $old_agile_sort_order_pos + 1]
    }
    default {
	ad_return_complaint 1 "Invalid action: $action"
    }
}


ns_log Notice "sprint-planner-action.tcl: after action switch: old_agile_status_pos=$old_agile_status_pos, old_agile_sort_order_pos=$old_agile_sort_order_pos"
ns_log Notice "sprint-planner-action.tcl: after action switch: new_agile_status_pos=$new_agile_status_pos, new_agile_sort_order_pos=$new_agile_sort_order_pos"

# ------------------------------------------------------------
# Check for movement beyond valid range
#
if {$new_agile_status_pos < 0} { set new_agile_status_pos 0 }
if {$new_agile_status_pos >= [llength $top_states_list]} { set new_agile_status_pos [expr [llength $top_states_list] - 1] }

if {$new_agile_sort_order_pos < 0} { set new_agile_sort_order_pos 0 }
if {$new_agile_sort_order_pos >= [llength $sprint_status_task_list]} { set new_agile_sort_order_pos [expr [llength $sprint_status_task_list] - 1]}

ns_log Notice "sprint-planner-action.tcl: after correction:    new_agile_status_pos=$new_agile_status_pos, new_agile_sort_order_pos=$new_agile_sort_order_pos"





# ------------------------------------------------------------
# Update status update (right/left)
#
if {$new_agile_status_pos ne $old_agile_status_pos} {
    set new_status_id [lindex $top_states_list $new_agile_status_pos]

    db_dml update_status "
	update im_timesheet_tasks
	set agile_status_id = :new_status_id
	where task_id = :old_task_id
    "
}


# ------------------------------------------------------------
# Update order update (up/down)
#
if {$old_agile_sort_order_pos ne $new_agile_sort_order_pos} {

    set new_task_id [lindex $sprint_status_task_list $new_agile_sort_order_pos]
    set new_sort_order $sort_order_hash($new_task_id)
    set old_sort_order $sort_order_hash($old_task_id)

    ns_log Notice "sprint-planner-action.tcl: before sort_order update: old_task_id=$old_task_id, new_task_id=$new_task_id, old_sort_order=$old_sort_order, new_sort_order=$new_sort_order"

    db_dml update_old_task_sort_order "
	update im_timesheet_tasks
	set agile_sort_order = :new_sort_order
	where task_id = :old_task_id
    "

    db_dml update_new_task_sort_order "
	update im_timesheet_tasks
	set agile_sort_order = :old_sort_order
	where task_id = :new_task_id
    "

}

ad_returnredirect $return_url
